@@ -111,13 +111,13 @@ class TestManageUserCommand(TestCase):
         User.objects.create(username=TEST_USERNAME, email=TEST_EMAIL)
         with pytest.raises(CommandError) as exc_context:
             call_command('manage_user', TEST_USERNAME, 'other@example.com')
-        assert 'email addresses do not match' in str(exc_context.value).lower()
+        assert 'do not match' in str(exc_context.value).lower()
         assert [(TEST_USERNAME, TEST_EMAIL)] == [(u.username, u.email) for u in User.objects.all()]
 
         # check that removal uses the same check
         with pytest.raises(CommandError) as exc_context:
             call_command('manage_user', TEST_USERNAME, 'other@example.com', '--remove')
-        assert 'email addresses do not match' in str(exc_context.value).lower()
+        assert 'do not match' in str(exc_context.value).lower()
         assert [(TEST_USERNAME, TEST_EMAIL)] == [(u.username, u.email) for u in User.objects.all()]
 
     def test_same_email_varied_case(self):
